@@ -8,31 +8,31 @@
  *****************************************************************/
 
 #include <chrono>
+#include <csignal>
 #include <cstdlib>
 #include <unistd.h>
 
+#include <QCommandLineParser>
+#include <QDBusConnection>
+#include <QDBusInterface>
+#include <QDBusReply>
+#include <QDebug>
+#include <QFile>
 #include <QIcon>
+#include <QTemporaryFile>
 #include <QTimer>
 
 #include <KAboutData>
 #include <KConfigGroup>
 #include <KLocalizedString>
 #include <KSharedConfig>
-#include <QCommandLineParser>
-#include <QDebug>
-#include <QFile>
-#include <QTemporaryFile>
-
-#include <QDBusConnection>
-#include <QDBusInterface>
-#include <QDBusReply>
-
-#include <config-drkonqi.h>
-
+#include <KSignalHandler>
 #ifdef Q_OS_MACOS
 #include <KWindowSystem>
 #endif
 
+#include <config-drkonqi.h>
+
 #include "backtracegenerator.h"
 #include "bugzillaintegration/reportinterface.h"
 #include "debuggermanager.h"
@@ -41,41 +41,40 @@
 #include "statusnotifier.h"
 
 using namespace std::chrono_literals;
+using namespace Qt::StringLiterals;
 
 static const char version[] = PROJECT_VERSION;
 
 namespace
 {
-// Clean on-disk data before quitting. This ought to only happen if we are
-// certain that the user doesn't want the crash to appear again.
-void cleanupAfterUserQuit()
-{
-    DrKonqi::cleanupBeforeQuit();
-    qApp->quit();
-}
 
 void aboutToQuit()
 {
-    if (ReportInterface::self()->hasCrashEventSent()) {
-        cleanupAfterUserQuit();
+    if (ReportInterface::self()->hasCrashEventSent() || DrKonqi::debuggerManager()->backtraceGenerator()->hasAnyFailure()) {
+        qApp->quit();
     } else {
         // Add a fallback timer. This timer makes sure that drkonqi will definitely quit, even if it should
         // have some sort of runtime defect that prevents reporting from finishing (and consequently not quitting).
         static QTimer timer;
         timer.setInterval(5min); // arbitrary time limit for trace+submission
-        QObject::connect(&timer, &QTimer::timeout, qApp, &cleanupAfterUserQuit);
-        QObject::connect(ReportInterface::self(), &ReportInterface::crashEventSent, qApp, &cleanupAfterUserQuit);
+        QObject::connect(&timer, &QTimer::timeout, qApp, &QCoreApplication::quit);
+        QObject::connect(ReportInterface::self(), &ReportInterface::crashEventSent, qApp, &QCoreApplication::quit);
+        QObject::connect(DrKonqi::debuggerManager()->backtraceGenerator(), &BacktraceGenerator::stateChanged, qApp, [] {
+            if (DrKonqi::debuggerManager()->backtraceGenerator()->hasAnyFailure()) {
+                qApp->quit();
+            }
+        });
         ReportInterface::self()->setSendWhenReady(true);
         timer.start();
     }
 }
 
-void openDrKonqiDialog()
+void openDrKonqiDialog(DrKonqiDialog::GoTo to = DrKonqiDialog::GoTo::Main)
 {
     auto *w = new DrKonqiDialog();
     QObject::connect(qApp, &QCoreApplication::aboutToQuit, w, &QObject::deleteLater);
     QObject::connect(qApp, &QGuiApplication::lastWindowClosed, qApp, &aboutToQuit);
-    w->show();
+    w->show(to);
 #ifdef Q_OS_MACOS
     KWindowSystem::forceActiveWindow(w->winId());
 #endif
@@ -100,7 +99,13 @@ void requestDrKonqiDialog(bool restarted, bool interactionAllowed)
         statusNotifier->notify(activation);
     }
     QObject::connect(statusNotifier, &StatusNotifier::expired, qApp, &aboutToQuit);
-    QObject::connect(statusNotifier, &StatusNotifier::activated, qApp, &openDrKonqiDialog);
+    QObject::connect(statusNotifier, &StatusNotifier::activated, qApp, [] {
+        openDrKonqiDialog(DrKonqiDialog::GoTo::Main);
+    });
+    QObject::connect(statusNotifier, &StatusNotifier::sentryActivated, qApp, [] {
+        qDebug() << "Sending report to sentry automatically";
+        openDrKonqiDialog(DrKonqiDialog::GoTo::Sentry);
+    });
 }
 
 bool isShuttingDown()
@@ -128,6 +133,15 @@ int main(int argc, char *argv[])
     QApplication app(argc, argv);
     KLocalizedString::setApplicationDomain(QByteArrayLiteral("drkonqi5"));
 
+    // We have somewhat special close behavior because of the internal debugger instance as well as auto-submission.
+    // Make sure to honor SIGINT in a timely manner.
+    KSignalHandler::self()->watchSignal(SIGINT);
+    QObject::connect(KSignalHandler::self(), &KSignalHandler::signalReceived, &app, [&app](int signal) {
+        if (signal == SIGINT) {
+            app.quit();
+        }
+    });
+
     // Prevent KApplication from setting the crash handler. We will set it later...
     setenv("KDE_DEBUG", "true", 1);
     // Session management is not needed, do not even connect in order to survive longer than ksmserver.
@@ -168,6 +182,9 @@ int main(int argc, char *argv[])
                                                      "the backtrace at startup"));
     const QCommandLineOption threadOption(QStringLiteral("thread"), i18nc("@info:shell", "The <thread id> of the failing thread"), QStringLiteral("threadid"));
     const QCommandLineOption dialogOption(QStringLiteral("dialog"), i18nc("@info:shell", "Do not show a notification but launch the debug dialog directly"));
+    const QCommandLineOption glRendererOption(u"glrenderer"_s, u"The GL_RENDERER used by the process"_s, u"glrenderer"_s);
+    const QCommandLineOption exceptionNameOption(u"exceptionname"_s, u"The exception class name if an exception was the cause"_s, u"name"_s);
+    const QCommandLineOption exceptionWhatOption(u"exceptionwhat"_s, u"The exception what string if an exception was the cause"_s, u"what"_s);
 
     parser.addOptions({signalOption,
                        appNameOption,
@@ -183,7 +200,10 @@ int main(int argc, char *argv[])
                        restartedOption,
                        keepRunningOption,
                        threadOption,
-                       dialogOption});
+                       dialogOption,
+                       glRendererOption,
+                       exceptionNameOption,
+                       exceptionWhatOption});
 
     // Add all unknown options but make sure to print a warning.
     // This enables older DrKonqi's to run by newer KCrash instances with
@@ -221,6 +241,21 @@ int main(int argc, char *argv[])
     DrKonqi::setKeepRunning(parser.isSet(keepRunningOption));
     DrKonqi::setThread(parser.value(threadOption).toInt());
     DrKonqi::setStartupId(parser.value(startupIdOption));
+    DrKonqi::instance()->m_glRenderer = parser.value(glRendererOption);
+    DrKonqi::instance()->m_exceptionName = parser.value(exceptionNameOption);
+    if (!DrKonqi::instance()->m_exceptionName.isEmpty()) {
+        QProcess proc;
+        proc.setProgram(u"c++filt"_s);
+        proc.setArguments({u"--types"_s, DrKonqi::instance()->m_exceptionName});
+        proc.start();
+        if (proc.waitForFinished((1000ms).count())) {
+            DrKonqi::instance()->m_exceptionName = QString::fromUtf8(proc.readAllStandardOutput().trimmed());
+        }
+        if (DrKonqi::instance()->m_exceptionName.isEmpty()) { // restore the potentially mangled version if empty
+            DrKonqi::instance()->m_exceptionName = parser.value(exceptionNameOption);
+        }
+    }
+    DrKonqi::instance()->m_exceptionWhat = parser.value(exceptionWhatOption);
     auto forceDialog = parser.isSet(dialogOption);
 
     if (!DrKonqi::init()) {
@@ -234,14 +269,12 @@ int main(int argc, char *argv[])
     const bool restarted = parser.isSet(restartedOption);
 
     // Whether the user should be encouraged to file a bug report
-    const bool interactionAllowed = KConfigGroup(KSharedConfig::openConfig(), "General").readEntry("InteractionAllowed", true);
+    const bool interactionAllowed = KConfigGroup(KSharedConfig::openConfig(), QStringLiteral("General")).readEntry("InteractionAllowed", true);
     const bool shuttingDown = isShuttingDown();
 
     if (forceDialog) {
         openDrKonqiDialog();
     } else if (shuttingDown) {
-        // this bypasses cleanupAfterUserQuit and consequently leaves the kcrash-metadata in place for later
-        // consumption via drkonqi-coredump-pickup
         return 0;
     } else {
         // if no notification service is running (eg. shell crashed, or other desktop environment)
