@@ -5,9 +5,7 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 #include "backtraceparser_p.h"
-#include "backtraceparsercdb.h"
 #include "backtraceparsergdb.h"
-#include "backtraceparserkdbgwin.h"
 #include "backtraceparserlldb.h"
 #include "backtraceparsernull.h"
 #include "drkonqi_parser_debug.h"
@@ -20,15 +18,11 @@ BacktraceParser *BacktraceParser::newParser(const QString &debuggerName, QObject
 {
     if (debuggerName == QLatin1String("gdb")) {
         return new BacktraceParserGdb(parent);
-    } else if (debuggerName == QLatin1String("kdbgwin")) {
-        return new BacktraceParserKdbgwin(parent);
-    } else if (debuggerName == QLatin1String("lldb")) {
-        return new BacktraceParserLldb(parent);
-    } else if (debuggerName == QLatin1String("cdb")) {
-        return new BacktraceParserCdb(parent);
-    } else {
-        return new BacktraceParserNull(parent);
     }
+    if (debuggerName == QLatin1String("lldb")) {
+        return new BacktraceParserLldb(parent);
+    }
+    return new BacktraceParserNull(parent);
 }
 
 BacktraceParser::BacktraceParser(QObject *parent)
@@ -94,19 +88,6 @@ BacktraceParser::Usefulness BacktraceParser::backtraceUsefulness() const
     return d ? d->m_usefulness : Useless;
 }
 
-QStringList BacktraceParser::firstValidFunctions() const
-{
-    Q_D(const BacktraceParser);
-
-    // if there is no cached usefulness, the data calculation function has not run yet.
-    if (d && d->m_usefulness == InvalidUsefulness) {
-        const_cast<BacktraceParser *>(this)->calculateRatingData();
-    }
-
-    // if there is no d, the debugger has not run yet, so we have no functions to return.
-    return d ? d->m_firstUsefulFunctions : QStringList();
-}
-
 QStringList BacktraceParser::librariesWithMissingDebugSymbols() const
 {
     Q_D(const BacktraceParser);
@@ -259,21 +240,6 @@ static bool isFunctionUseful(const BacktraceLine &line)
     return true;
 }
 
-static bool isFunctionUsefulForSearch(const BacktraceLine &line)
-{
-    // Ignore Qt containers (and iterators Q*Iterator)
-    if (line.functionName().startsWith(QLatin1String("QList")) || line.functionName().startsWith(QLatin1String("QLinkedList"))
-        || line.functionName().startsWith(QLatin1String("QVector")) || line.functionName().startsWith(QLatin1String("QStack"))
-        || line.functionName().startsWith(QLatin1String("QQueue")) || line.functionName().startsWith(QLatin1String("QSet"))
-        || line.functionName().startsWith(QLatin1String("QMap")) || line.functionName().startsWith(QLatin1String("QMultiMap"))
-        || line.functionName().startsWith(QLatin1String("QMapData")) || line.functionName().startsWith(QLatin1String("QHash"))
-        || line.functionName().startsWith(QLatin1String("QMultiHash")) || line.functionName().startsWith(QLatin1String("QHashData"))) {
-        return false;
-    }
-
-    return true;
-}
-
 void BacktraceParser::calculateRatingData()
 {
     Q_D(BacktraceParser);
@@ -329,7 +295,6 @@ void BacktraceParser::calculateRatingData()
 
     i.toFront(); // Reuse the list iterator
     int functionIndex = 0;
-    int usefulFunctionsCount = 0;
     bool firstUsefulFound = false;
     while (i.hasNext() && functionIndex < 5) {
         const BacktraceLine &line = i.next();
@@ -340,12 +305,6 @@ void BacktraceParser::calculateRatingData()
             // Save simplified backtrace line
             d->m_simplifiedBacktrace += line.toString();
 
-            // Fetch three useful functions (only functionName) for search queries
-            if (usefulFunctionsCount < 3 && isFunctionUsefulForSearch(line) && !d->m_firstUsefulFunctions.contains(line.functionName())) {
-                d->m_firstUsefulFunctions.append(line.functionName());
-                usefulFunctionsCount++;
-            }
-
             functionIndex++;
         } else if (firstUsefulFound) {
             // Add "[...]" if there are invalid functions in the middle
